<?php
/*
*** PostgreSQL
create table loadavg (
	id integer primary key default nextval('seq'),
	time integer not null,
	load integer
);
create index idx_loadavg_time on loadavg (time);

*** MySQL
CREATE TABLE tx_phploadavg_log (
	uid int(11) NOT NULL auto_increment,
	pid int(11) DEFAULT '0' NOT NULL,
	tstamp int(11) DEFAULT '0' NOT NULL,
	crdate int(11) DEFAULT '0' NOT NULL,
	cruser_id int(11) DEFAULT '0' NOT NULL,
	deleted tinyint(4) DEFAULT '0' NOT NULL,
	hidden tinyint(4) DEFAULT '0' NOT NULL,
	loadavg tinytext NOT NULL,

	PRIMARY KEY (uid),
	KEY parent (pid)
);
*/

ini_set('display_errors', TRUE);
error_reporting(E_ALL ^ E_NOTICE);

define('PLA_STORAGE_POSTGRES', 'PLA_STORAGE_POSTGRES');
define('PLA_STORAGE_MYSQL', 'PLA_STORAGE_MYSQL');
define('PLA_STORAGE_SESSION', 'PLA_STORAGE_SESSION');

//require_once("../nadlib/class.HTMLForm.php");
require_once("class.slTable.php");

class pla_sysinfo {

	function getCPU() {
		$output = file("/proc/cpuinfo");
		$printed = FALSE;
		$times = 0;
		foreach($output as $line) {
			if (FALSE !== strpos($line, "model name")) {
				if (!$printed) {
					$ret = substr($line, strpos($line, ": ")+2);
					$ret = trim($ret);
					$printed = TRUE;
				}
				$times++;
			}
		}
		if ($times > 1) {
			$ret .= ' (x'.$times.')';
		}
		return $ret;
	}

	function getMemory() {
		$output = file("/proc/meminfo");
		foreach($output as $line) {
			if (FALSE !== strpos($line, "MemTotal")) {
				$retM = round(intval(substr($line, strpos($line, ": ")+2))/1024) . ' MB';
			}
			if (FALSE !== strpos($line, "MemFree")) {
				$retF = round(intval(substr($line, strpos($line, ": ")+2))/1024) . ' MB';
			}
		}
		return $retF . '/' . $retM . '=' . round($retF*100/$retM) . '%';
	}

	function getUptime() {
		$output = implode("", file("/proc/uptime"));
		$uptime = explode(" ", $output);
		$uptime = $uptime[0];
		$uptime = floor($uptime / (3600*24)) . " days " .
		str_pad(floor($uptime % (3600*24) / 3600), 2, '0', STR_PAD_LEFT) .
		":" . str_pad(floor(($uptime % (3600*24) % 3600) / 60), 2, '0', STR_PAD_LEFT);
		return $uptime;
	}

	function getFreeSpace() {
		return
		number_format(disk_free_space('/')/1024/1024/1024, 2).' GB/'.
		number_format(disk_total_space('/')/1024/1024/1024, 2).' GB='.
		number_format(disk_free_space('/')/disk_total_space('/')*100, 2).'%';
	}

	function getHTTPCons() {
		$connect = 0;
		$http = 0;
		$output = file('/proc/net/tcp');
		foreach ($output as $line) {
			$cols = split(' +', $line);
			list($ip, $port) = split(':', $cols[2]);
			if (hexdec($port) == 443) {
				$http++;
				if ($cols[4] == '0A') {
					$connect++;
				}
			}
		}
		return $connect.'/'.$http;
	}

	function getClientDate() {
		return '
		<script>
		<!--
			function pad(a) {
				if (a.toString().length < 2) {
					return "0"+a;
				} else {
					return a;
				}
			}
			var date = new Date();
			var d = pad(date.getDate());
			var m = pad(date.getMonth()+1);
			var Y = pad(date.getFullYear());
			var H = pad(date.getHours());
			var i = pad(date.getMinutes());
			var s = pad(date.getSeconds());
			document.write(d+"."+m+"."+Y+" "+H+":"+i+":"+s);
		// -->
		</script>';
	}

	function getAdminButtons() {
/*		$f = new HTMLForm();
		$f->method('GET');
		$f->target('stats');
		$f->hidden('get', 3);
		$f->submit('Stats');
		$content .= $f->getContent();
*/

		$content .= '<form target="stats"><input type="hidden" name="get" value="3"><input type="submit" value="Stats"></form>';
		return $content;
	}

	function getStats() {
		$stats = array();
		$stats['Host'] = $_SERVER['HTTP_HOST'] .'/'. $_SERVER['SERVER_NAME'] .'/'. $_SERVER['SERVER_ADDR'];
		$stats['Uptime'] = $this->getUptime();
		$stats['CPU'] = $this->getCPU();
		$stats['Mem Free'] = $this->getMemory();
		$stats['HDD Free'] = $this->getFreeSpace();
		//$stats['HTTPs'] = getHTTPCons();
		$stats['SysTime'] = date('d.m.Y H:i:s');
		$stats['CliTime'] = $this->getClientDate();
//		$stats['Admin'] = $this->getAdminButtons();
		return $stats;
	}

}

abstract class phploadavg {
	var $refresh_timeout = 5; // seconds
	var $chartLength = 1000;
	var $chartHeight = 200;

	function __construct() {
	}

	function getLoadAvg() {
		return floatval(implode('', array_slice(explode(' ', @file_get_contents('/proc/loadavg')), 0, 1)));
	}

	abstract function insertLoadAvg($load);

	abstract function getLoadData();

	abstract function showLoadAvg($da, $max);

	function getMaxAverage($da) {
		$max = 0;
		$sum = 0;
		foreach ($da as $time => $load) {
			$max = max($max, $load);
			$sum += $load;
		}
		return array($da, $max, $sum / sizeof($da));
	}

	function main($load) {
		$this->insertLoadAvg($load);
		list($da, $max, $avg) = $this->getLoadData();
		print('<table class="folder" style="width: '.$this->chartLength.'px;"><tr>');
			print('<td class="caption">');
				print("Current load: ".number_format($load, 2).
						", Max: ".number_format($max, 2).
						", Avg: ".number_format($avg, 2).'<br>'
				);
			print('</td><td class="captionSpace">&nbsp;</td></tr>');
			print('<tr><td class="surface" colspan="2">');
				$this->showLoadAvg($da, $max);
			print('</td></tr>');
		print('</table>');
		$this->showAvgLine($da, $max, $avg);

/*		$sObj = new pla_sysinfo();
		$stats = $sObj->getStats();
		$this->showStats($stats);
*/	}

	function showStats($stats) {
		$s = new slTable(NULL, 'style="position: absolute; left: 4px; top: 34px;"');
		$s->thes(array(
			'name' => array('name' => '', 'no_hsc' => TRUE),
			'value' => array('name' => '', 'no_hsc' => TRUE),
		));
		foreach ($stats as $name => $val) {
			$s->addRowData(array('name' => $name.':&nbsp;', 'value' => $val));
		}
		$s->render();
	}

	function showStatsXML($stats) {
		header('Content-type: text/xml');
		println('<?xml version="1.0"?>');
		println('<phploadavg>');
		foreach ($stats as $name => $val) {
			println("\t<$name>".htmlspecialchars($val)."</$name>");
		}
		println('</phploadavg>');
	}

} // class phploadavg

abstract class phploadavg_table extends phploadavg {

	function showLoadAvg($da, $max) {
		$print .= ('<table class="columnContainer"><tr>');
		list($firstTime, $firstLoad) = each($da);
		$prevDate = $firstTime - ($firstTime % 3600);
		foreach ($da as $time => $load) {
			++$i;
			$print .= ('<td>');

			if (!$max) {
				$h = 0;
			} else {
				$h = round($load / $max * 100);
			}

			// for firefox
			if ($h == 0) {
				$h = 1;
			}

			$class = "fill";
			if ($load/100 > 1) $class = "attention";
			if ($load/100 > 4) $class = "overload";

			$date = $time - ($time % 3600);
			if ($date != $prevDate) {
				$class = "midnight";
				$h = 100;
				$divContent .= '<div style="position: relative; left: '.$i.'px; top: 0px">'.date("H:i", $date+3600).'</div>';
			}
			$prevDate = $date;
			$print .= ('<table class="bottom"><tr><td class="white"><div style="width: 1px; height: 1px"></div></td></tr>');
			$print .= ('<tr><td class="'.$class.'" style="height: '.$h.'%;"><img src="clear.gif" width="1" height="1" /></td></tr></table>');

			$print .= ('</td>');
		}
		$print .= ('</tr></table>');
		print($divContent);
		print($print);
	}

	function showAvgLine($da, $max, $avg) {
		if ($max) {
			print('<table style="position: relative; left: 3px; top: '.(20-$avg/$max*$this->chartHeight).'px; height: 1px;"');
			print('<tr><td class="overload"><img src="clear.gif" width="'.($this->chartLength-6).'" height="1" /></td></tr>');
			print('</table>');
		}
	}

} // class phploadavg_table

abstract class phploadavg_gif extends phploadavg {
	var $pixelsPerReading = 1;

	/**
	 * Overloaded, because the whole table is not needed in this case as only GIF is reloaded, we need to put the text of loadavg into the GIF
	 *
	 * @param unknown_type $load
	 */
	function main($load) {
		$this->insertLoadAvg($load);
		$this->showLoadAvg(NULL, NULL); // <img src...>
	}

	function showLoadAvg($da, $max) {
		$content .= '<img src="index.php?id='.$GLOBALS['TSFE']->id.'&tx_phploadavg_pi1[gif]=1" width="'.$this->chartLength.'" height="'.$this->chartHeight.'" />';
		print($content);
	}

	function renderGIF($load) {
		list($da, $max, $avg) = $this->getLoadData();

		// create
		$im = imagecreate($this->chartLength, $this->chartHeight);
		$background_color = imagecolorallocate($im, 0x00, 0x22, 0x00);

		// grid
		$grid_color = imagecolorallocate($im, 0, 0x44, 0);
		for ($i = 0; $i < $this->chartLength; $i += 10) {
			imageline($im, $i, 0, $i, $this->chartHeight, $grid_color);
		}
		for ($i = 0; $i < $this->chartHeight; $i += 10) {
			imageline($im, 0, $i, $this->chartLength, $i, $grid_color);
		}

		// chart
		$chart_color = imagecolorallocate($im, 0, 0xBB, 0);
		if ($this->pixelsPerReading == 0) {
			$i = 0;
			foreach ($da as $time => $load) {
				$y = $this->getY($load, $max);
				imageline($im, $i, $y, $i, $this->chartHeight, $chart_color);
				$i++;
			}
		} else {
			reset($da);
			$current = current($da);
			$da2 = array_slice($da, 1, $this->chartLength);
			$x1 = 0;
			$y1 = $this->getY($current, $max);
			foreach ($da2 as $time => $load) {
				$x2 = $x1 + $this->pixelsPerReading;
				$y2 = $this->getY($load, $max);
				imageline($im, $x1, $y1, $x2, $y2, $chart_color);
				$x1 = $x2;
				$y1 = $y2;
				if ($x1 > $this->chartLength) {
					break;
				}
			}
		}

		// hour lines
		$hour_color = imagecolorallocate($im, 0x88, 0x88, 0x88);
		reset($da);
		list($firstTime, $firstLoad) = each($da);
		$prevDate = $firstTime - ($firstTime % 3600);
		$x = 0;
		foreach ($da as $time => $load) {
			$date = $time - ($time % 3600);
			if ($date != $prevDate) {
				$y = $this->getY($load, $max);
				imageline($im, $x, $y, $x, $this->chartHeight, $hour_color);
				imagestring($im, 4, $x, $this->chartHeight - 15, date('H:i', $date+3600), $hour_color);
			}
			$prevDate = $date;
			$x++;
		}

		// text
		$text_color = imagecolorallocate($im, 0, 0xBB, 0);
		$shadow_color = imagecolorallocate($im, 0, 80, 0);
		$text = "Current load: ".number_format($load, 2).", Max: ".number_format($max, 2).", Avg: ".number_format($avg, 2);//.', '.sizeof($da);
		imagestring($im, 4, 5, 4,  $text, $shadow_color);
		imagestring($im, 4, 5, 5,  $text, $shadow_color);
		imagestring($im, 4, 4, 5,  $text, $shadow_color);
		imagestring($im, 4, 3, 4,  $text, $shadow_color);
		imagestring($im, 4, 4, 3,  $text, $shadow_color);
		imagestring($im, 4, 3, 3,  $text, $shadow_color);
		imagestring($im, 4, 4, 4,  $text, $text_color);

		// average
		$avg_color = imagecolorallocate($im, 0xBB, 0xBB, 0);
		$avgY = $this->getY($avg, $max);
		imageLine($im, 0, $avgY, $this->chartLength, $avgY, $avg_color);

		// render
		header("Content-type: image/png");
		imagepng($im);
		imagedestroy($im);
		exit();
	}

	function getY($load, $max) {
		return $max ? $this->chartHeight - ($load / $max * $this->chartHeight) : 0;
	}

}

class phploadavg_postgres extends phploadavg_table {

	function __construct() {
		parent::__construct();
		require_once("../class.dbLayer.php");
		if ($_SERVER['SERVER_ADDR'] == "10.31.1.11") {
			$postgres_db = "phploadavg";
			$postgres_login = "slawa";
			$postgres_password = "slawadb";
		} else {
			$postgres_db = "slawa";
			$postgres_login = "slawa";
			$postgres_password = "slawa";
		}
		$GLOBALS['db'] = new dbLayer($postgres_db, $postgres_login, $postgres_password);
	}

	function insertLoadAvg($load) {
		$qu = "select * from loadavg order by id desc limit 1";
		$res = $GLOBALS['db']->perform($qu);
		$row = pg_fetch_assoc($res);
		$time = time();
		if ($time - $row['time'] > $this->refresh_timeout) {
			$load *= 100;
			$load = round($load);
			$qu = "insert into loadavg (time, load) values ('$time', '$load')";
			$GLOBALS['db']->perform($qu);
		}
	}

	function getLoadData() {
		$qu = "select load, time from loadavg order by time desc limit ".$this->chartLength;
		$da = $GLOBALS['db']->getTableDataSql($qu);

		$assoc = array();
		foreach ($da as $d) {
			$assoc[$d['time']] = $d['load'] / 100;
		}
		return $this->getMaxAverage($assoc);
	}

	function getAvgFromDB($date, $last, $plus) {
		$rows = array();
		$stop = FALSE;
		while (!$stop) {
			$till = strtotime($plus, $date);
			$query = "select avg(load) from loadavg where time between '".$date."' and '".$till."'";
			$avg = $GLOBALS['db']->getFirstValue($query);
			$avg = number_format($avg / 100, 2);
			$row = array(
				'time' => $date,
				'date' => date('d.m.Y', $date),
				'load' => $avg,
			);
			$rows[] = $row;

			$date = $till;
			if ($date > $last) {
				$stop = TRUE;
			}
		}
		return $rows;
	}

} // class phploadavg_postgres

class phploadavg_mysql extends phploadavg_gif {

	function __construct() {
		parent::__construct();
		//mysql_connect(...);
	}

	function insertLoadAvg($load) {
		$qu = "select * from tx_phploadavg_log order by uid desc limit 1";
		$res = mysql_query($qu);
		$row = mysql_fetch_assoc($res);
		$time = time();

		// to prevent two people watching the same chart generate twich amount of data
		if ($time - $row['time'] > $this->refresh_timeout) {
			$qu = "insert into tx_phploadavg_log (crdate, loadavg) values ('$time', '$load')";
			mysql_query($qu);
		}
	}

	function getLoadData() {
		$query = "select loadavg, crdate from tx_phploadavg_log order by crdate desc limit ".($this->chartLength + 40); // WHY?!? otherwise the chart is not full! I don't know why
		$res = mysql_query($query);
		$da = $this->fetchAll($res);

		$assoc = array();
		foreach ($da as $d) {
			$assoc[$d['crdate']] = $d['loadavg'];
		}
		return $this->getMaxAverage($assoc);
	}

	function fetchAll($res) {
		$rows = array();
		while (($row = mysql_fetch_assoc($res)) !== FALSE) {
			$rows[] = $row;
		}
		return $rows;
	}

	function getAvgFromDB($date, $last, $plus) {
		$rows = array();
		return $rows;
	}

} // class phploadavg_mysql

class phploadavg_session extends phploadavg_table {

	function __construct() {
		parent::__construct();
		session_start();
		if (!$_SESSION[__CLASS__]) {
			$_SESSION[__CLASS__] = array();
		}
	}

	function insertLoadAvg($load) {
		$keys = array_keys($_SESSION[__CLASS__]);
		$last = end($keys);
		//print(time() . ' ' . $last . ' ' . date('H:i:s', time()) . ' ' . date('H:i:s', $last));
		if (time() - $last > $this->refresh_timeout) {
			$_SESSION[__CLASS__][time()] = $load;
		}
	}

	function getLoadData() {
		$assoc = $_SESSION[__CLASS__];
		//$assoc = array_slice($assoc, 0, $this->chartLength, TRUE);
		$assoc = array_slice($assoc, -$this->chartLength, $this->chartLength, TRUE);
		$assoc = array_reverse($assoc, TRUE);
		//print('<pre>');		var_dump($assoc);		print('</pre>');
		return $this->getMaxAverage($assoc);
	}

	function getAvgFromDB($date, $last, $plus) {
		return array();
	}

} // class phploadavg_session

class pla_stats extends phploadavg_session {

	function addPercentTable($row, $avgM) {
		$avgP = round(100*$row['load']/$avgM);
		//if ($avgP == 0) $avgP = 1;
		if ($avgP == 100) $avgP = 99;
		$row['graph'] = '<table width=100%><tr><td style="width: '.$avgP.'%" class="fill"><img src="clear.gif" width="1" height="10"></td><td style="width: '.(100-$avgP).'%" aclass="white"><img src="clear.gif" width="1" height="10"></td></table>';
		return $row;
	}

	function showAvgStats() {
		$thes = array('date' => 'date', 'load' => 'load', 'graph' => array('thes' => 'graph', 'more' => 'width="85%"'));

		$query = "select min(time) from loadavg";
		$date = $GLOBALS['db']->getFirstValue($query);
		$date = mktime(0, 0, 0, date('m', $date), 1, date('Y', $date));

		$query = "select max(load) from loadavg";
		$avgM = $GLOBALS['db']->getFirstValue($query) / 100 / 10;
//		printbr($avgM);

		$rows = getAvgFromDB($date, time(), '+1 month');
//		$avgM = max(array_column($rows, 'load'));
		$s = new slTable(NULL, 'class="loadTable" width="100%"');
		$s->sortable = TRUE;
		$s->thes($thes);
		foreach ($rows as $row) {
			$row = $this->addPercentTable($row, $avgM);
			$s->addRowData($row);
		}
		$s->render();

		foreach ($rows as $row) {
			$days = getAvgFromDB($row['time'], strtotime('+1 month', $row['time']), '+1 day');
//			$avgD = max(array_column($days, 'load'));
			$s = new slTable(NULL, 'class="loadTable" width="100%"');
			$s->sortable = TRUE;
			$s->thes($thes);
			foreach($days as $day) {
				$day = $this->addPercentTable($day, $avgM);
				$s->addRowData($day);
			}
			$s->render();
		}
	}

} // class pla_stats

/************************* MAIN *************************/

class pla_factory {
	var $pla; 		// phploadavg descendant

	function outputHTMLHead() {
		print('
		<META HTTP-EQUIV="Page-Enter" CONTENT="BlendTrans(Duration=1)">
		<META HTTP-EQUIV="Site-Exit" CONTENT="BlendTrans(Duration=1)">
		<link rel="stylesheet" type="text/css" href="loadavg.css">');
	}

	function main() {
		header("Pragma: no-cache");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");

		$this->pla = new phploadavg_mysql();
		$load = $this->pla->getLoadAvg();
		//print($load); exit();
		//$load = rand(0, 2000)/1000;
		if ($_REQUEST['get'] == 1) {
			echo $load;
		} else if ($_REQUEST['get'] == 2) {
			$stats = array();
			$stats['Load'] = $load;//round($load*1000);
			$stats = $stats + $this->pla->getStats();
			$this->pla->showStatsXML($stats);
		} else if ($_REQUEST['get'] == 3) {
			$this->outputHTMLHead();
			$plastat = new pla_stats();
			$plastat->showAvgStats();
		} else if ($_REQUEST['tx_phploadavg_pi1']['gif']) {
			$this->pla->renderGIF($load);
		} else {
			$this->outputHTMLHead();
			$this->pla->main($load);
			$this->outputRefreshScript();
		}
	}

	function outputRefreshScript() {
		?>
		<script>
			function refr() {
				location.href='index.php?refresh='+Math.random();
			}
			setTimeout("refr()", <?=$this->pla->refresh_timeout * 1000?>);
		</script>
		<?php
	}

}

//$pla = new pla_factory();
//$pla->main();

?>